#!/usr/bin/env python
# coding: utf-8
"""
Dummy module for testing out tasks.
"""

from __future__ import print_function

import datetime

from siskin.task import DefaultTask


class DummyTask(DefaultTask):
    """
    A base task in the dummy tag space.
    """
    TAG = 'dummy'


class DummyHelloWorld(DummyTask):
    """
    Just say hello world.
    """

    def run(self):
        print("Hello from %s" % self.__class__.__name__)

    def complete(self):
        return False


class DummyFail(DummyTask):
    """
    A task that always fails.
    """

    def run(self):
        raise RuntimeError('This error occured at %s' % datetime.datetime.now())

    def complete(self):
        return False



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
OECD, refs #11575

Config

[oecd]

url = http://example.com/sample.xml
"""

import datetime

import luigi

from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class OECDTask(DefaultTask):
    """ OECD base task. """
    TAG = '52'

    def closest(self):
        return monthly(date=self.date)


class OECDDownload(OECDTask):
    """
    Download from a single URL.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("curl -sL --fail {url} > {output}", url=self.config.get('oecd', 'url'))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))


class OECDMARC(OECDTask):
    """
    Convert to importable MARC.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return OECDDownload()

    def run(self):
        output = shellout("python {script} {input} {output}",
                          script=self.assets('52/52_marcbinary.py'),
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
mediarep.org, refs #14064.
"""

import datetime
import os

import luigi

from gluish.common import Executable
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class MediarepTask(DefaultTask):
    """
    Base task.
    """
    TAG = '170'

    def closest(self):
        return monthly(date=self.date)


class MediarepMARC(MediarepTask):
    """
    Harvest and convert to MARC.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="https://mediarep.org/oai/request", significant=False)
    prefix = luigi.Parameter(default="oai_dc", significant=False)

    def requires(self):
        return Executable(name='metha-sync', message='https://github.com/miku/metha'),

    def run(self):
        shellout("METHA_DIR={dir} metha-sync -format {prefix} {url}",
                 prefix=self.prefix,
                 url=self.url,
                 dir=self.config.get('core', 'metha-dir'))
        data = shellout("""METHA_DIR={dir} metha-cat -root Records -format {prefix} {url} > {output}""",
                        dir=self.config.get('core', 'metha-dir'),
                        prefix=self.prefix,
                        url=self.url)
        output = shellout("""python {script} {input} {output}""",
                          script=self.assets('170/170_marcbinary.py'),
                          input=data)
        luigi.LocalTarget(output).move(self.output().path)

        os.remove(data)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="fincmarc.mrc"))


class MediarepIntermediateSchema(MediarepTask):
    """
    Single file dump.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="https://mediarep.org/oai/request", significant=False)
    prefix = luigi.Parameter(default="dim", significant=False)

    def requires(self):
        return [
            Executable(name='metha-sync', message='https://github.com/miku/metha'),
            Executable(name='span-import', message='https://github.com/miku/span'),
        ]

    def run(self):
        shellout("METHA_DIR={dir} metha-sync -format {prefix} {url}",
                 prefix=self.prefix,
                 url=self.url,
                 dir=self.config.get('core', 'metha-dir'))
        output = shellout("""METHA_DIR={dir} metha-cat -root Records -format {prefix} {url} |
                             span-import -i mediarep-dim | pigz -c > {output}""",
                          prefix=self.prefix,
                          url=self.url,
                          dir=self.config.get('core', 'metha-dir'))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="ldj.gz"))



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Cambridge University Press Journals

refs #10167.

Config:

[cambridge]

scp-src = user@ftp.online:/home/cambridge

"""

import datetime
import os

import luigi

from gluish.format import TSV
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.common import Executable
from siskin.task import DefaultTask
from siskin.utils import iterfiles


class CambridgeTask(DefaultTask):
    """
    Base task.
    """
    TAG = '133'

    def closest(self):
        return weekly(date=self.date)


class CambridgeDropbox(CambridgeTask):
    """
    Pull down content from FTP dropbox, in Dec '18 about 10K zips.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return Executable('rsync', message='https://rsync.samba.org/')

    def run(self):
        target = os.path.join(self.taskdir(), 'mirror')
        shellout("mkdir -p {target} && rsync {rsync_options} {src} {target}",
                 rsync_options=self.config.get('cambridge', 'rsync-options', fallback='-avzP'),
                 src=self.config.get('cambridge', 'scp-src'),
                 target=target)

        if not os.path.exists(self.taskdir()):
            os.makedirs(self.taskdir())

        with self.output().open('w') as output:
            for path in iterfiles(target):
                output.write_tsv(path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='filelist'), format=TSV)



# coding: utf-8
# pylint: disable=E1101,C0111
#
# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Heidelberger historische Bestände digital, #5964.
"""

import datetime

import luigi

from gluish.format import Gzip
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class HHBDTask(DefaultTask):
    """ Base task for source. """
    TAG = '107'

    def closest(self):
        return monthly(self.date)


class HHBDCombine(HHBDTask):
    """
    OAI, single file version.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    format = luigi.Parameter(default='oai_dc')
    url = luigi.Parameter(default='http://digi.ub.uni-heidelberg.de/cgi-bin/digioai.cgi', significant=False)

    def run(self):
        shellout("metha-sync -format {format} http://digi.ub.uni-heidelberg.de/cgi-bin/digioai.cgi", format=self.format)
        output = shellout("metha-cat -format {format} -root Records {url} > {output}", format=self.format, url=self.url)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))


class HHBDIntermediateSchema(HHBDTask):
    """
    Convert to intermediate schema via metafacture.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return HHBDCombine(date=self.date, format='oai_dc')

    def run(self):
        output = shellout("span-import -i hhbd < {input} | pigz -c > {output}", input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'), format=Gzip)


class HHBDExport(HHBDTask):
    """
    Export and hard-wire label, DE-540.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    format = luigi.Parameter(default='solr5vu3')

    def requires(self):
        return HHBDIntermediateSchema(date=self.date)

    def run(self):
        """ Tag by hand since not yet in AMSL. """
        output = shellout("""
	    unpigz -c {input} | span-tag -c '{{"DE-540": {{"any": {{}}}}}}' |
            span-export -with-fullrecord -o {format} > {output}
        """,
                          format=self.format,
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))



# coding: utf-8
# pylint: disable=C0103,W0232,C0301,W0703

# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Static mappings for reuse
"""

from collections import defaultdict

formats = defaultdict(dict)
languages = {}
roles = {}

#################################################################################
# Formate
#################################################################################

formats["Unknown"]["Leader"] = "     xxx  22        4500"
formats["Unknown"]["p007"] = "tu"
formats["Unknown"]["e007"] = "cr"
formats["Unknown"]["008"] = ""
formats["Unknown"]["336b"] = ""
formats["Unknown"]["338b"] = ""
formats["Unknown"]["655a"] = ""
formats["Unknown"]["6552"] = ""
formats["Unknown"]["935c"] = ""

formats["Multipart"]["Leader"] = "     cam  22       a4500"
formats["Multipart"]["p007"] = "tu"
formats["Multipart"]["e007"] = "cr"
formats["Multipart"]["008"] = ""
formats["Multipart"]["336b"] = ""
formats["Multipart"]["338b"] = ""
formats["Multipart"]["655a"] = ""
formats["Multipart"]["6552"] = ""
formats["Multipart"]["935c"] = ""

formats["Kit"]["Leader"] = "     com  22       a4500"
formats["Kit"]["p007"] = "ou"
formats["Kit"]["e007"] = "ou"
formats["Kit"]["008"] = ""
formats["Kit"]["336b"] = ""
formats["Kit"]["338b"] = ""
formats["Kit"]["655a"] = ""
formats["Kit"]["6552"] = ""
formats["Kit"]["935c"] = ""

formats["Manuscript"]["Leader"] = "     ctm  22        4500"
formats["Manuscript"]["p007"] = "tu"
formats["Manuscript"]["e007"] = "cr"
formats["Manuscript"]["008"] = ""
formats["Manuscript"]["336b"] = "txt"
formats["Manuscript"]["338b"] = "nc"
formats["Manuscript"]["655a"] = ""
formats["Manuscript"]["6552"] = ""
formats["Manuscript"]["935c"] = ""

#formats["Braille"]["Leader"] = ""
#formats["Braille"]["p007"] = ""
#formats["Braille"]["e007"] = ""
#formats["Braille"]["008"] = ""
#formats["Braille"]["336b"] = ""
#formats["Braille"]["338b"] = ""
#formats["Braille"]["655a"] = ""
#formats["Braille"]["6552"] = ""
#formats["Braille"]["935c"] = ""

formats["Image"]["Leader"] = "     ckm  22        4500"
formats["Image"]["p007"] = "kv"
formats["Image"]["e007"] = "cr"
formats["Image"]["008"] = ""
formats["Image"]["336b"] = "sti"
formats["Image"]["338b"] = "no"
formats["Image"]["655a"] = ""
formats["Image"]["6552"] = ""
formats["Image"]["935c"] = ""

formats["Thesis"]["Leader"] = "     cam  22        4500"
formats["Thesis"]["p007"] = "tu"
formats["Thesis"]["e007"] = "cr"
formats["Thesis"]["008"] = ""
formats["Thesis"]["336b"] = "txt"
formats["Thesis"]["338b"] = "nc"
formats["Thesis"]["655a"] = "Hochschulschrift"
formats["Thesis"]["6552"] = "gnd-content"
formats["Thesis"]["935c"] = ""

formats["Score"]["Leader"] = "     cam  22        4500"
formats["Score"]["p007"] = "tu"
formats["Score"]["e007"] = "cr"
formats["Score"]["008"] = ""
formats["Score"]["336b"] = ""
formats["Score"]["338b"] = ""
formats["Score"]["655a"] = ""
formats["Score"]["6552"] = ""
formats["Score"]["935c"] = "muno"

formats["Microform"]["Leader"] = "     cam  22        4500"
formats["Microform"]["p007"] = "hu"
formats["Microform"]["e007"] = "hu"
formats["Microform"]["008"] = ""
formats["Microform"]["336b"] = "txt"
formats["Microform"]["338b"] = "hj"
formats["Microform"]["655a"] = ""
formats["Microform"]["6552"] = ""
formats["Microform"]["935c"] = ""

formats["Map"]["Leader"] = "     cem  22        4500"
formats["Map"]["p007"] = "au"
formats["Map"]["e007"] = "cr"
formats["Map"]["008"] = ""
formats["Map"]["336b"] = "sti"
formats["Map"]["338b"] = "nc"
formats["Map"]["655a"] = "Atlas"
formats["Map"]["6552"] = "gnd-content"
formats["Map"]["935c"] = ""

formats["Book"]["Leader"] = "     cam  22        4500"
formats["Book"]["p007"] = "tu"
formats["Book"]["e007"] = "cr"
formats["Book"]["008"] = ""
formats["Book"]["336b"] = "txt"
formats["Book"]["338b"] = "nc"
formats["Book"]["655a"] = ""
formats["Book"]["6552"] = ""
formats["Book"]["935c"] = ""

formats["Series"]["Leader"] = "     cas  22        4500"
formats["Series"]["p007"] = "tu"
formats["Series"]["e007"] = "cr"
formats["Series"]["008"] = ""
formats["Series"]["336b"] = ""
formats["Series"]["338b"] = ""
formats["Series"]["655a"] = ""
formats["Series"]["6552"] = ""
formats["Series"]["935c"] = ""

formats["Newspaper"]["Leader"] = "     cas  22        4500"
formats["Newspaper"]["p007"] = "tu"
formats["Newspaper"]["e007"] = "cr"
formats["Newspaper"]["008"] = "n"
formats["Newspaper"]["336b"] = ""
formats["Newspaper"]["338b"] = ""
formats["Newspaper"]["655a"] = ""
formats["Newspaper"]["6552"] = ""
formats["Newspaper"]["935c"] = "zt"

formats["Journal"]["Leader"] = "     cas  22        4500"
formats["Journal"]["p007"] = "tu"
formats["Journal"]["e007"] = "cr"
formats["Journal"]["008"] = "p"
formats["Journal"]["336b"] = ""
formats["Journal"]["338b"] = ""
formats["Journal"]["655a"] = ""
formats["Journal"]["6552"] = ""
formats["Journal"]["935c"] = ""

formats["Article"]["Leader"] = "     cab  22        4500"
formats["Article"]["p007"] = "tu"
formats["Article"]["e007"] = "cr"
formats["Article"]["008"] = ""
formats["Article"]["336b"] = ""
formats["Article"]["338b"] = ""
formats["Article"]["655a"] = ""
formats["Article"]["6552"] = ""
formats["Article"]["935c"] = ""

formats["Chapter"]["Leader"] = "     caa  22        4500"
formats["Chapter"]["p007"] = "tu"
formats["Chapter"]["e007"] = "cr"
formats["Chapter"]["008"] = ""
formats["Chapter"]["336b"] = ""
formats["Chapter"]["338b"] = ""
formats["Chapter"]["655a"] = ""
formats["Chapter"]["6552"] = ""
formats["Chapter"]["935c"] = ""

formats["Loose-leaf"]["Leader"] = "     cai  22        4500"
formats["Loose-leaf"]["p007"] = "td"
formats["Loose-leaf"]["e007"] = "td"
formats["Loose-leaf"]["008"] = ""
formats["Loose-leaf"]["336b"] = ""
formats["Loose-leaf"]["338b"] = ""
formats["Loose-leaf"]["655a"] = "Loseblattsammlung"
formats["Loose-leaf"]["6552"] = "gnd-carrier"
formats["Loose-leaf"]["935c"] = "lo"

formats["Audio-Cassette"]["Leader"] = "     cjm  22        4500"
formats["Audio-Cassette"]["e007"] = "ss"
formats["Audio-Cassette"]["p007"] = "ss"
formats["Audio-Cassette"]["008"] = ""
formats["Audio-Cassette"]["336b"] = "prm"
formats["Audio-Cassette"]["338b"] = "ss"
formats["Audio-Cassette"]["655a"] = ""
formats["Audio-Cassette"]["6552"] = ""
formats["Audio-Cassette"]["935c"] = "muto"

formats["Video-Cassette"]["Leader"] = "     cgm  22        4500"
formats["Video-Cassette"]["e007"] = "vf"
formats["Video-Cassette"]["p007"] = "vf"
formats["Video-Cassette"]["008"] = ""
formats["Video-Cassette"]["336b"] = "tdi"
formats["Video-Cassette"]["338b"] = "vf"
formats["Video-Cassette"]["655a"] = "Film"
formats["Video-Cassette"]["6552"] = "gnd-content"
formats["Video-Cassette"]["935c"] = "vide"

formats["Blu-Ray-Disc"]["Leader"] = "     cgm  22        4500"
formats["Blu-Ray-Disc"]["e007"] = "vd"
formats["Blu-Ray-Disc"]["p007"] = "vd"
formats["Blu-Ray-Disc"]["008"] = ""
formats["Blu-Ray-Disc"]["336b"] = "tdi"
formats["Blu-Ray-Disc"]["338b"] = "vd"
formats["Blu-Ray-Disc"]["655a"] = "Blu-Ray-Disc"
formats["Blu-Ray-Disc"]["6552"] = "gnd-carrier"
formats["Blu-Ray-Disc"]["935c"] = "vide"

formats["DVD-Video"]["Leader"] = "     cgm  22        4500"
formats["DVD-Video"]["e007"] = "co"
formats["DVD-Video"]["p007"] = "co"
formats["DVD-Video"]["008"] = ""
formats["DVD-Video"]["336b"] = "tdi"
formats["DVD-Video"]["338b"] = "vd"
formats["DVD-Video"]["655a"] = "DVD-Video"
formats["DVD-Video"]["6552"] = "gnd-carrier"
formats["DVD-Video"]["935c"] = "vide"

formats["DVD-Audio"]["Leader"] = "     cjm  22        4500"
formats["DVD-Audio"]["p007"] = "co"
formats["DVD-Audio"]["e007"] = "co"
formats["DVD-Audio"]["008"] = ""
formats["DVD-Audio"]["336b"] = "prm"
formats["DVD-Audio"]["338b"] = "sd"
formats["DVD-Audio"]["655a"] = "DVD-Audio"
formats["DVD-Audio"]["6552"] = "gnd-carrier"
formats["DVD-Audio"]["935c"] = "muto"

formats["DVD-ROM"]["Leader"] = "     cam  22        4500"
formats["DVD-ROM"]["p007"] = "co"
formats["DVD-ROM"]["e007"] = "co"
formats["DVD-ROM"]["008"] = ""
formats["DVD-ROM"]["336b"] = "txt"
formats["DVD-ROM"]["338b"] = "dvd"
formats["DVD-ROM"]["655a"] = "DVD-ROM"
formats["DVD-ROM"]["6552"] = "gnd-carrier"
formats["DVD-ROM"]["935c"] = ""

formats["CD-Video"]["Leader"] = "     cgm  22        4500"
formats["CD-Video"]["p007"] = "co"
formats["CD-Video"]["e007"] = "co"
formats["CD-Video"]["008"] = ""
formats["CD-Video"]["336b"] = "tdi"
formats["CD-Video"]["338b"] = "vd"
formats["CD-Video"]["655a"] = "CD"
formats["CD-Video"]["6552"] = "gnd-carrier"
formats["CD-Video"]["935c"] = "vide"

formats["CD-Audio"]["Leader"] = "     cjm  22        4500"
formats["CD-Audio"]["p007"] = "sd"
formats["CD-Audio"]["e007"] = "sd"
formats["CD-Audio"]["008"] = ""
formats["CD-Audio"]["336b"] = "prm"
formats["CD-Audio"]["338b"] = "sd"
formats["CD-Audio"]["655a"] = "CD"
formats["CD-Audio"]["6552"] = "gnd-carrier"
formats["CD-Audio"]["935c"] = "muto"

formats["CD-ROM"]["Leader"] = "     cam  22        4500"
formats["CD-ROM"]["p007"] = "co"
formats["CD-ROM"]["e007"] = "co"
formats["CD-ROM"]["008"] = ""
formats["CD-ROM"]["336b"] = "txt"
formats["CD-ROM"]["338b"] = "cd"
formats["CD-ROM"]["655a"] = ""
formats["CD-ROM"]["6552"] = "gnd-carrier"
formats["CD-ROM"]["935c"] = ""

formats["Floppy-Disk"]["Leader"] = "     cam  22        4500"
formats["Floppy-Disk"]["p007"] = "cj"
formats["Floppy-Disk"]["e007"] = "cj"
formats["Floppy-Disk"]["008"] = ""
formats["Floppy-Disk"]["336b"] = ""
formats["Floppy-Disk"]["338b"] = ""
formats["Floppy-Disk"]["655a"] = "Diskette"
formats["Floppy-Disk"]["6552"] = "gnd-carrier"
formats["Floppy-Disk"]["935c"] = ""

formats["Online-Video"]["Leader"] = "     ngm  22        4500"
formats["Online-Video"]["p007"] = "cr"
formats["Online-Video"]["e007"] = "cr"
formats["Online-Video"]["008"] = ""
formats["Online-Video"]["336b"] = "tdi"
formats["Online-Video"]["338b"] = "cr"
formats["Online-Video"]["655a"] = "Film"
formats["Online-Video"]["6552"] = "gnd-content"
formats["Online-Video"]["935c"] = "vide"

formats["Online-Audio"]["Leader"] = "     cam  22        4500"
formats["Online-Audio"]["p007"] = "cr"
formats["Online-Audio"]["e007"] = "cr"
formats["Online-Audio"]["008"] = ""
formats["Online-Audio"]["336b"] = "prm"
formats["Online-Audio"]["338b"] = "cr"
formats["Online-Audio"]["655a"] = ""
formats["Online-Audio"]["6552"] = ""
formats["Online-Audio"]["935c"] = "muto"

formats["Website"]["Leader"] = "     cai  22        4500"
formats["Website"]["p007"] = "cr"
formats["Website"]["e007"] = "cr"
formats["Website"]["008"] = ""
formats["Website"]["336b"] = "txt"
formats["Website"]["338b"] = "cr"
formats["Website"]["655a"] = "Website"
formats["Website"]["6552"] = "gnd-content"
formats["Website"]["935c"] = ""

formats["Object"]["Leader"] = "     crm  22        4500"
formats["Object"]["p007"] = "zz"
formats["Object"]["e007"] = "zz"
formats["Object"]["008"] = ""
formats["Object"]["336b"] = ""
formats["Object"]["338b"] = "nr"
formats["Object"]["655a"] = ""
formats["Object"]["6552"] = ""
formats["Object"]["935c"] = "gegenst"

#################################################################################
# Sprachen
#################################################################################

languages["english"] = "eng"
languages["french"] = "fre"
languages["german"] = "ger"
languages["greek"] = "gre"
languages["italian"] = "ita"
languages["latin"] = "lat"
languages["polish"] = "pol"
languages["russian"] = "rus"
languages["sanskrit"] = "san"
languages["spanish"] = "spa"

languages["ar"] = "ara"
languages["bo"] = "tib"
languages["cs"] = "cze"
languages["da"] = "dan"
languages["de"] = "ger"
languages["el"] = "gre"
languages["en"] = "eng"
languages["es"] = "spa"
languages["fr"] = "fre"
languages["he"] = "heb"
languages["hi"] = "hin"
languages["it"] = "ita"
languages["la"] = "lat"
languages["nl"] = "dut"
languages["pl"] = "pol"
languages["ru"] = "rus"
languages["sa"] = "san"
languages["sp"] = "spa"
languages["ur"] = "urd"
languages["yi"] = "yid"

#################################################################################
# Personenrollen
#################################################################################

roles["author"] = "aut"
roles["auteur du texte"] = "aut"
roles["chorégraphe"] = "chr"
roles["composer"] = "cmp"
roles["compositeur"] = "cmp"
roles["darst"] = "act"
roles["darsteller"] = "act"
roles["donateur"] = "dnr"
roles["drehbuch"] = "aus"
roles["editor"] = "edt"
roles["éditeur scientifique"] = "edt"
roles["hrsg"] = "edt"
roles["kamera"] = "vdg"
roles["komp"] = "cmp"
roles["musik"] = "cmp"
roles["prod"] = "pro"
roles["produzent"] = "pro"
roles["regie"] = "drt"
roles["schnitt"] = "edm"
roles["szenenbild"] = "prs"
roles["übers"] = "trl"



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
https://www.ceeol.com/, refs #9398.

Currently, there are two XML formats used, a custom CEEOL XML and a slightly
invalid MARCXML. The last update was manually uploaded to a colorful server
@Telekom.

Config:

[ceeol]

disk-dir = /path/to/disk
updates = /path/to/file.xml, /path/to/another.xml, ...
updates-marc = /path/to/file.xml, /path/to/file.xml

"""

from __future__ import print_function

import glob
import hashlib
import json
import os
import re
import sys
import tempfile

import luigi

import marcx
import pymarc
from gluish.utils import shellout
from siskin.task import DefaultTask


class CeeolTask(DefaultTask):
    """
    Base task.
    """
    TAG = '53'

    def updateid(self):
        """
        Only use the string of files as hashed identifier. TODO(miku): It would
        be more robust to use the checksums of all input files.
        """
        sha1 = hashlib.sha1()
        sha1.update(self.config.get("ceeol", "updates").encode('utf-8'))
        sha1.update(self.config.get("ceeol", "updates-marc").encode('utf-8'))
        return sha1.hexdigest()


class CeeolJournalsUpdates(CeeolTask):
    """
    Create an intermediate schema from zero or more update XML (in MARCXML).
    The output will depend on the set and order of the input files.
    """

    def run(self):
        paths = [p.strip() for p in self.config.get("ceeol", "updates").split(",") if p.strip()]
        _, stopover = tempfile.mkstemp(prefix="siskin-")
        self.logger.debug("found %d updates", len(paths))
        for p in paths:
            shellout("span-import -i ceeol-marcxml {input} >> {output}", input=p, output=stopover)

        # Append MARC updates.
        paths = [p.strip() for p in self.config.get("ceeol", "updates-marc").split(",") if p.strip()]
        _, stopover = tempfile.mkstemp(prefix="siskin-")
        self.logger.debug("found %d updates (MARC)", len(paths))
        with open(stopover, 'a') as output:
            for p in paths:
                self.logger.debug("converting: %s", p)
                for doc in convert_ceeol_to_intermediate_schema(p):
                    output.write(json.dumps(doc) + "\n")
        luigi.LocalTarget(stopover).move(self.output().path)

    def output(self):
        filename = "{}.ndj".format(self.updateid())
        return luigi.LocalTarget(path=self.path(filename=filename))


class CeeolJournalsIntermediateSchema(CeeolTask):
    """
    Combine all journals from disk dump and convert them to intermediate
    schema. Add updates. The output name of this task depends on the update id.

    This task depends on a fixed subdirectory, e.g. articles or 1Journal-Articles-XML..
    """

    def requires(self):
        return CeeolJournalsUpdates()

    def run(self):
        diskdir = self.config.get('ceeol', 'disk-dir')
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        for path in glob.glob(os.path.join(diskdir, 'articles', 'articles_*xml')):
            shellout("span-import -i ceeol {input} | pigz -c >> {output}", input=path, output=stopover)
        shellout("cat {update} | pigz -c >> {output}", update=self.input().path, output=stopover)
        luigi.LocalTarget(stopover).move(self.output().path)

    def output(self):
        filename = "{}.ldj.gz".format(self.updateid())
        return luigi.LocalTarget(path=self.path(filename=filename))


# Helper for MARC, XXX: maybe move this out.

_362a_pattern = re.compile(r"vol[.]?[ ]*([0-9]+)?,[ ]*no[.][ ]*([0-9]+)[ ]*\(([0-9]+)\)-?", re.IGNORECASE)


def format_date(value, iso=False):
    """
    Try to get a suitable date out of the value.
    """
    if len(value) == 4 and value.isnumeric():
        if iso:
            return "{}-01-01T00:00:00Z".format(value)
        else:
            return "{}-01-01".format(value)

    raise ValueError("could not handle: %s", value)


def clean_260a(value):
    """
    Strip place value.
    """
    return value.replace("[1] :", "").strip()


def parse_volume_from_362a(value, default=None):
    """
    Try to parse volume from 362a.

    Example value: Vol. 1, no. 113 (2006)-
    """
    if value is None:
        return default
    match = _362a_pattern.search(value)
    if not match:
        return default
    return match.group(1).strip() if match.group(1) else default


def parse_issue_from_362a(value, default=None):
    """
    Try to parse volume from 362a.

    Example value: Vol. 1, no. 113 (2006)-
    """
    if value is None:
        return default
    match = _362a_pattern.search(value)
    if not match:
        return default
    return match.group(2).strip() if match.group(1) else default


def convert_ceeol_to_intermediate_schema(filename):
    """
    Given an XML filename, convert data to intermediate schema. Note: The XML
    refers to the partially broken XML supplied as of 07/2019.

    File must fit into memory.

    Yields converted dictionaries.
    """
    article_url_re = re.compile(r"https://www.ceeol.com/search/article-detail\?id=([0-9]+)")

    with open(filename) as handle:
        records = pymarc.marcxml.parse_xml_to_array(handle)
        for record in records:
            record = marcx.Record.from_record(record)

            # Try to find ID.
            for url in record.itervalues("856.u"):
                match = article_url_re.search(url)
                if not match:
                    continue
                record_id = match.group(1)
                break
            else:
                raise ValueError("missing record id")

            doc = {
                "abstract": record.firstvalue("520.a", default=""),
                "authors": [{
                    "rft.auname": v
                } for v in record.itervalues("100.a")],
                "finc.format": "ElectronicArticle",
                "finc.id": "ai-53-{}".format(record_id),
                "finc.mega_collection": ["CEEOL Central and Eastern European Online Library"],
                "finc.record_id": record_id,
                "finc.source_id": "53",
                "languages": list(record.itervalues("041.a")),
                "rft.atitle": record.firstvalue("245.a", default=""),
                "rft.date": format_date(record.firstvalue("260.c", default="")),
                "rft.genre": "article",
                "rft.issn": list(record.itervalues("022.a")),
                "rft.issue": parse_issue_from_362a(record.firstvalue("362.a", default=""), default=""),
                "rft.place": [clean_260a(v) for v in record.itervalues("260.a")],
                "rft.pub": list(record.itervalues("260.b")),
                "rft.volume": parse_volume_from_362a(record.firstvalue("362.a", default=""), default=""),
                "ris.type": "EJOUR",
                "subjects": list(record.itervalues("650.a")),
                "url": list(record.itervalues("856.u")),
                "x.date": format_date(record.firstvalue("260.c", default=""), iso=True),
            }
            yield doc



# coding: utf-8
# pylint: disable=C0103,W0232,C0301,W0703

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Define a siskin wide task with artifacts under core.home directory.

[core]

smtp = server.example.com
default-sender = my@mail.com
default-replyto = my@mail.com
error-email = a@c.com, d@e.com
home = /path/to/dir

[amsl]

write-url = https://live.abc.technology/w/i/write

"""

import datetime
import logging
import os
import re
import socket
import tempfile
import traceback

import luigi

from gluish.task import BaseTask
from gluish.utils import shellout
from siskin import __version__
from siskin.configuration import Config
from siskin.mail import send_mail

config = Config.instance()


class DefaultTask(BaseTask):
    """
    Base task for all siskin tasks.

    It sets the base directory, where all task artifacts will be stored. It
    also provides shortcuts to config, assets and logging objects.

    On failure, an email is sent to configured addresses.

    A command line parameter named --stamp is used to optionally update
    timestamps in AMSL electronic resource management system.
    """
    BASE = config.get('core', 'home', fallback=os.path.join(tempfile.gettempdir(), 'siskin-data'))

    stamp = luigi.BoolParameter(default=False,
                                description="update processing time of source via AMSL API",
                                significant=False)

    def assets(self, path):
        """
        Return the absolute path to the asset. `path` is the relative path
        below the assets root dir.
        """
        return os.path.join(os.path.dirname(__file__), 'assets', path)

    @property
    def config(self):
        """
        Return the config instance.
        """
        return config

    @property
    def logger(self):
        """
        Return the logger. Module logging uses singleton internally, so no worries.
        """
        return logging.getLogger('siskin')

    def on_failure(self, exception):
        """
        If a task fails, try to send an email.
        """
        try:
            tolist = self.config.get("core", "error-email").split(",")
            subject = "%s %s" % (self, datetime.datetime.today().strftime("%Y-%m-%d %H:%M"))
            message = """
            This is siskin {version} on {host}.

            An error occured in Task {name}, this is the error message:

            {exc}

            Stacktrace:

            {tb}
            """.format(
                version=__version__,
                name=self,
                exc=exception,
                tb=traceback.format_exc(),
                host=socket.gethostname(),
            )
            message = message.encode("utf-8")
            send_mail(tolist=tolist, subject=subject, message=message)
            self.logger.debug("sent error emails to %s", ", ".join(tolist))
        except TypeError as err:
            self.logger.debug("error-email may not be configured, not sending mail: %s", err)
        except Exception as err:
            self.logger.debug("failed to send error email: %s", err)

    def on_success(self):
        """
        Try to send a datestamp to AMSL, but only if a couple of prerequisites are met:

        All subclasses inherit a --stamp boolean flag, which must be set. If
        the TAG of the source is not numeric, we won't do anything. If
        "amsl.write-url" configuration is not set, we will log the error, but
        do not stop processing. Finally, even if the HTTP request fails, it
        won't be fatal.

        On success, the API returns:

            < HTTP/1.1 200 OK
            < Content-Type: text/html; charset=UTF-8
            < Content-Length: 2
            ...

            OK

        Note that if a subclass overwrites `on_success` this method is not
        called, so you have to call it manually.
        """
        if not self.stamp:
            return
        if not hasattr(self, 'TAG'):
            self.logger.warn("no tag defined, skip stamping")
            return
        if not re.match(r"^[\d]+$", self.TAG):
            self.logger.warn("non-integer source id: %s, skip stamping", self.TAG)
            return

        sid = self.TAG.lstrip("0")  # Otherwise: Parameter 'sid' ... not a positive integer.

        try:
            write_url = config.get("amsl", "write-url")
            if write_url is None:
                self.logger.warn("missing amsl.write-url configuration, skip stamping")
                return
        except Exception as err:
            self.logger.warn("could not stamp: %s", err)
            return

        try:
            shellout("""curl --fail -XPOST "{write_url}?do=updatetime&sid={sid}" > /dev/null """,
                     write_url=write_url,
                     sid=sid)
        except RuntimeError as err:
            self.logger.warn(err)
            return
        else:
            self.logger.debug("successfully stamped: %s", sid)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Pubmed PMC FTP.
"""

from __future__ import print_function

import datetime

import luigi

from gluish.format import TSV
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.benchmark import timed
from siskin.common import FTPMirror
from siskin.task import DefaultTask


class PubmedTask(DefaultTask):
    """ Pubmed base. """
    TAG = 'pubmed'

    def closest(self):
        return weekly(self.date)


class PubmedMetadataPaths(PubmedTask):
    """
    Sync metadata only (much faster).
    """
    date = ClosestDateParameter(default=datetime.date.today())
    max_retries = luigi.IntParameter(default=10, significant=False)
    timeout = luigi.IntParameter(default=20, significant=False, description='timeout in seconds')

    def requires(self):
        return FTPMirror(host='ftp.ncbi.nlm.nih.gov',
                         base='/pub/pmc/',
                         pattern='articles*tar.gz',
                         max_retries=self.max_retries,
                         timeout=self.timeout)

    @timed
    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class PubmedJournalList(PubmedTask):
    """
    Download journal list.
    """

    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout(
            """curl --fail "http://www.ncbi.nlm.nih.gov/pmc/journals/collections/?format=csv" > {output} """)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path())



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Springer (testing).

Options currently: Download via OwnCloud and (preferred) SFTP.

Config
------

[springer]

username = admin
password = s3cret
url = http://export.com/intermediate.file.gz

ftp-host = sftp://example.com
ftp-base = /target_data/xyz
ftp-username= admin
ftp-password= s3cr3t
ftp-pattern = *

"""

from __future__ import print_function

import datetime
import re

import luigi
import six
import json

from gluish.format import TSV, Gzip
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.common import FTPMirror
from siskin.decorator import deprecated
from siskin.sources.amsl import AMSLFilterConfig
from siskin.task import DefaultTask


class SpringerTask(DefaultTask):
    TAG = '105'

    def closest(self):
        return weekly(date=self.date)


class SpringerPaths(SpringerTask):
    """
    Mirror SLUB FTP. Preferred as of November 2017.
    """

    date = ClosestDateParameter(default=datetime.date.today())
    max_retries = luigi.IntParameter(default=10, significant=False)
    timeout = luigi.IntParameter(default=20, significant=False, description='timeout in seconds')

    def requires(self):
        return FTPMirror(host=self.config.get('springer', 'ftp-host'),
                         base=self.config.get('springer', 'ftp-base'),
                         username=self.config.get('springer', 'ftp-username'),
                         password=self.config.get('springer', 'ftp-password'),
                         pattern=self.config.get('springer', 'ftp-pattern'),
                         max_retries=self.max_retries,
                         timeout=self.timeout)

    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class SpringerCleanup(SpringerTask):
    """
    2017-11-28: finc.mega_collection is now multi-valued; AIAccessFacet remains.
    2017-12-12: new finc.id, refs #11821, #11960, #11961.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return SpringerPaths(date=self.date)

    def run(self):
        realpath = None
        with self.input().open() as handle:
            for row in handle.iter_tsv(cols=('path', )):
                if not row.path.endswith("total_tpu.ldj.gz"):
                    continue
                realpath = row.path
                break
            else:
                raise RuntimeError('FTP site does not contain total_tpu.ldj.gz')
        output = shellout("""
            unpigz -c {input} | jq -rc 'del(.["finc.AIRecordType"]) | del(.["AIAccessFacet"])' |
            jq -c '. + {{ "finc.record_id": .doi, "finc.format": "ElectronicArticle", "url": ["https://doi.org/" + .doi] }}' | pigz -c > {output}
        """,
                          input=realpath)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'), format=Gzip)


class SpringerIntermediateSchema(SpringerTask, luigi.WrapperTask):
    """
    Just the cleaned version.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return SpringerCleanup(date=self.date)

    def output(self):
        return self.input()



#!/usr/bin/env python3
# coding: utf-8

# Quelle: Deutsches Textarchiv
# SID: 44
# Tickets: #14972
# technicalCollectionID: sid-44-col-textarchiv
# Task: TODO

import base64
import io
import os
import re
import sys

import xmltodict

import marcx
from siskin.utils import marc_build_imprint

inputfilename = "44_input.xml"
outputfilename = "44_output.mrc"

if len(sys.argv) >= 3:
    inputfilename, outputfilename = sys.argv[1:3]

inputfile = io.open(inputfilename, "rb")
outputfile = io.open(outputfilename, "wb")

inputfile = open(inputfilename, "rb")
xmlfile = inputfile.read()
records = xmltodict.parse(xmlfile,
                          force_list=("cmdp:title", "cmdp:author", "cmdp:editor", "cmdp:pubPlace", "cmdp:publisher",
                                      "cmdp:date"))
outputfile = open(outputfilename, "wb")

for xmlrecord in records["Records"]["Record"]:

    if not xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:fileDesc"].get("cmdp:titleStmt"):
        continue

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False

    # Leader
    marcrecord.leader = "     nam  22        4500"

    # Identifikator
    id = xmlrecord["header"]["identifier"]
    id = bytes(id)
    id = base64.b64encode(id)
    f001 = id.decode("utf-8").rstrip("=")
    marcrecord.add("001", data="finc-44-" + f001)

    # Zugangsformat
    marcrecord.add("007", data="cr")

    # Sortierjahr
    imprint = xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:fileDesc"]["cmdp:sourceDesc"][
        "cmdp:biblFull"].get("cmdp:publicationStmt", "")
    if imprint:
        year = imprint["cmdp:date"][0]["#text"]
        match = re.match("\d\d\d\d", year)
        if match:
            marcrecord.add("008", data="0000000%s" % year)

    # Sprache
    marcrecord.add("041", a="ger")

    # 1. Urheber
    authors = xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:fileDesc"][
        "cmdp:titleStmt"].get("cmdp:author", "")
    if authors:
        surname = authors[0]["cmdp:persName"].get("cmdp:surname", "")
        forename = authors[0]["cmdp:persName"].get("cmdp:forename", "")
        rolename = authors[0]["cmdp:persName"].get("cmdp:roleName", "")
        if rolename and forename:
            rolename = " <" + rolename + ">"
            f100a = forename + rolename
        elif surname and forename:
            f100a = surname + ", " + forename
        elif surname and surname != "N. N.":
            f100a = surname
        else:
            f100a = ""
        marcrecord.add("100", a=f100a, e="verfasserin", _4="aut")

    # Titlel
    titles = xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:fileDesc"]["cmdp:titleStmt"][
        "cmdp:title"]
    f245a = titles[0]["#text"]
    f245b = ""
    f245n = ""
    if len(titles) == 2:
        for title in titles:
            if "#text" in title:
                f245b = title["#text"]
            if "@n" in title:
                f245n = title["@n"]
    marcrecord.add("245", a=f245a, b=f245b, n=f245n)

    # Ausgabe
    edition = xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:fileDesc"].get(
        "cmdp:editionStmt", "")
    if edition:
        f250a = edition["cmdp:edition"]
        marcrecord.add("250", a=f250a)

    # Erscheinungsvermerk
    imprint = xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:fileDesc"]["cmdp:sourceDesc"][
        "cmdp:biblFull"].get("cmdp:publicationStmt", "")
    if imprint:
        f260a = ""
        f260b = ""
        f260c = ""
        place = imprint.get("cmdp:pubPlace")
        if place:
            f260a = place[0]
        publisher = imprint.get("cmdp:publisher")
        if publisher:
            f260b = publisher[0]["cmdp:name"]
            if not f260b:  # sometimes exists but None
                f260b = ""
        f260c = imprint["cmdp:date"][0]["#text"]
        subfields = marc_build_imprint(f260a, f260b, f260c)
        marcrecord.add("260", subfields=subfields)

    # Umfangsangabe
    pages = xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:fileDesc"]["cmdp:sourceDesc"][
        "cmdp:biblFull"].get("cmdp:extent", "")
    if pages:
        f300a = pages["cmdp:measure"]["#text"]
        marcrecord.add("300", a=f300a)

    # Inhaltsbeschreibung
    abstract = xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:profileDesc"].get(
        "cmdp:abstract", "")
    if abstract:
        f520a = abstract["cmdp:p"]
        marcrecord.add("520", a=f520a)

    # Schlagwörter
    subjects = xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:profileDesc"].get(
        "cmdp:textClass", "")
    if subjects:
        for subject in subjects["cmdp:classCode"]:
            subject = subject["#text"]
            if subject[0].isupper():
                subjects = re.split("[;,]", subject)
                for subject in subjects:
                    f650a = subject.strip()
                    marcrecord.add("650", a=f650a)

    # Weitere Urheber
    authors = xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:fileDesc"][
        "cmdp:titleStmt"].get("cmdp:author", "")
    if authors:
        for author in authors[1:]:
            surname = author["cmdp:persName"].get("cmdp:surname", "")
            forename = author["cmdp:persName"].get("cmdp:forename", "")
            rolename = author["cmdp:persName"].get("cmdp:roleName", "")
            if rolename and forename:
                rolename = " <" + rolename + ">"
                f700a = forename + rolename
            elif surname and forename:
                f700a = surname + ", " + forename
            elif surname and surname != "N. N.":
                f700a = surname
            else:
                f700a = ""
            marcrecord.add("700", a=f700a, e="verfasserin", _4="aut")

    # Herausgeber
    editors = xmlrecord["metadata"]["cmd:CMD"]["cmd:Components"]["cmdp:teiHeader"]["cmdp:fileDesc"][
        "cmdp:titleStmt"].get("cmdp:editor", "")
    if authors:
        for author in authors[1:]:
            surname = author["cmdp:persName"].get("cmdp:surname", "")
            forename = author["cmdp:persName"].get("cmdp:forename", "")
            if surname and forename:
                f700a = surname + ", " + forename
            elif surname and surname != "N. N.":
                f700a = surname
            else:
                f700a = ""
            marcrecord.add("700", a=f700a, e="hrsg", _4="edt")

    # Link zur Ressource
    urls = xmlrecord["metadata"]["cmd:CMD"]["cmd:Resources"]["cmd:ResourceProxyList"]["cmd:ResourceProxy"]
    xml = urls[0]["cmd:ResourceRef"]
    html = urls[1]["cmd:ResourceRef"]
    txt = urls[2]["cmd:ResourceRef"]
    page = urls[3]["cmd:ResourceRef"]
    marcrecord.add("856", q="text/xml", _3="Download als XML", u=xml)
    marcrecord.add("856", q="text/html", _3="Download als HTML", u=html)
    marcrecord.add("856", q="text/plain", _3="Download als Text", u=txt)
    marcrecord.add("856", q="text/html", _3="Link zur Seite", u=page)

    # Format
    marcrecord.add("935", b="cofz")

    # Kollektion
    collections = ["a", f001, "b", "44", "c", "sid-44-col-textarchiv"]
    marcrecord.add("980", subfields=collections)

    outputfile.write(marcrecord.as_marc())

outputfile.close()



#!/usr/bin/env python3
# coding: utf-8

import re
import sys

import marcx
import pymarc

copytags = ("003", "005", "006", "007", "008", "020", "040", "043", "050", "082", "100", "245", "246", "250", "264",
            "300", "336", "337", "338", "490", "500", "505", "520", "600", "610", "650", "651", "700", "710", "776",
            "830", "856")

inputfilename = "161_input.mrc"
outputfilename = "161_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")
reader = pymarc.MARCReader(inputfile)

for oldrecord in reader:

    try:
        f245a = oldrecord["245"]["a"]
    except:
        continue

    newrecord = marcx.Record(force_utf8=True)

    # Leader
    leader = "     " + oldrecord.leader[5:]
    newrecord.leader = leader

    # Identifikator
    f001 = oldrecord["001"].data
    f001 = str(f001)
    newrecord.add("001", data="finc-161-" + f001)

    # Originalfelder
    for tag in copytags:

        # ISBN
        if tag == "020":

            try:
                f020a = oldrecord["020"]["a"]
            except:
                f020a = ""

            if f020a != "":
                f020a = f020a.replace(" ", "-")
                f020a = f020a.replace(".", "-")
                regexp = re.search("([0-9xX-]{10,17})", f020a)

                if regexp:
                    f020a = regexp.group(1)
                    f020a = f020a.rstrip("-")
                    newrecord.add("020", a=f020a)
                else:
                    print("Die ISBN %s konnte nicht mittels regulärer Ausdrücke überprüft werden." % f020a)

        else:
            for field in oldrecord.get_fields(tag):
                newrecord.add_field(field)

    # Ansigelung
    collections = ["a", f001, "b", "161", "c", "sid-161-col-cambridgeopenaccess"]
    newrecord.add("980", subfields=collections)

    outputfile.write(newrecord.as_marc())

inputfile.close()
outputfile.close()



# coding: utf-8
# pylint: disable=F0401,W0232,E1101,C0103,C0301,W0223,E1123,R0904,E1103

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
"""
Directory of Open Access Journals.

DOAJ is an online directory that indexes and provides access to high quality,
open access, peer-reviewed journals.

http://doaj.org
"""

import datetime
import itertools
import operator
import os
import tempfile
import time
from builtins import map, range

import elasticsearch
import luigi
import json

from gluish.common import Executable
from gluish.format import TSV, Gzip
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.benchmark import timed
from siskin.task import DefaultTask
from siskin.utils import load_set_from_file, load_set_from_target


class DOAJTask(DefaultTask):
    """
    Base task for DOAJ.
    """
    TAG = '28'

    def closest(self):
        return monthly(date=self.date)


class DOAJHarvest(DOAJTask):
    """
    Harvest via OAI endpoint.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout(
            "METHA_DIR={dir} metha-sync {endpoint} && METHA_DIR={dir} metha-cat {endpoint} | pigz -c > {output}",
            dir=self.config.get('core', 'metha-dir'),
            endpoint='http://www.doaj.org/oai.article')
        self.logger.debug("compressing output (might take a few minutes) ...")
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml.gz'))


class DOAJIntermediateSchemaDirty(DOAJTask):
    """
    Convert to intermediate schema via span; elasticsearch and API are
    currently defunkt, use OAI (with metha 0.1.37 or higher).

    Dirty version, contains dups (like https://is.gd/AJ0qKc).

    Example duplicates:

    * https://doaj.org/article/697e7128445f406fa108acf55a3d0b52
    * https://doaj.org/article/fb7aa986929e4644b6125214bade736f

    Example seems to differ only in abstract formatting (https://i.imgur.com/ILxmRVC.png), with
    fb7aa986929e4644b6125214bade736f probably being correct (also the newest
    record).

    2011-01-01T00:00:00Z    5a29b0fd51bf458eaa173f5c40f3d21c        A CLINICAL EXPERIENCE OF METHOTREXATE USE IN TREATMENT OF PATIENT WITH JUVENILE OLIGOARTHRITIS
    2011-01-01T00:00:00Z    697e7128445f406fa108acf55a3d0b52        A CLINICAL EXPERIENCE OF METHOTREXATE USE IN TREATMENT OF PATIENT WITH JUVENILE OLIGOARTHRITIS
    2011-01-01T00:00:00Z    fb7aa986929e4644b6125214bade736f        A CLINICAL EXPERIENCE OF METHOTREXATE USE IN TREATMENT OF PATIENT WITH JUVENILE OLIGOARTHRITIS
    2011-11-01T00:00:00Z    fb7aa986929e4644b6125214bade736f        A CLINICAL EXPERIENCE OF METHOTREXATE USE IN TREATMENT OF PATIENT WITH JUVENILE OLIGOARTHRITIS

    Another example (DOAJ internal search seems to return both: https://is.gd/AJ0qKc):

    2006-09-01T00:00:00Z    e211856315f442869421032269b461d6        11 de setembro de 2001: algumas das consequências após 5 anos.
    2010-12-01T00:00:00Z    d8e93099d0934e6eb3f4f66e8e522a9c        11 de setembro de 2001: algumas das consequências após 5 anos.

    """
    date = ClosestDateParameter(default=datetime.date.today())
    format = luigi.Parameter(default="doaj-oai",
                             description="kind of source document, doaj-oai (defunkt: doaj, doaj-api)")

    def requires(self):
        return {
            'span-import': Executable(name='span-import', message='http://git.io/vI8NV'),
            'input': DOAJHarvest(date=self.date),
        }

    @timed
    def run(self):
        output = shellout("""unpigz -c {input} |
                             span-import -i {format} |
                             grep -vf {exclude} |
                             pigz -c > {output}""",
                          exclude=self.assets('028_doaj_filter_issn.tsv'),
                          input=self.input().get('input').path,
                          format=self.format)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'))


class DOAJTable(DOAJTask):
    """
    Emit a DOAJ data table: id, date, title - for rough deduplication by title (see DOAJWhitelist).
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return DOAJIntermediateSchema(date=self.date)

    def run(self):
        output = shellout(
            """unpigz -c {input} | jq -r '[.["finc.record_id"], .["x.date"], .["rft.atitle"]] | @tsv' > {output} """,
            input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class DOAJWhitelist(DOAJTask):
    """
    Create a list of suppressable DOAJ ids.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return DOAJTable(date=self.date)

    def run(self):
        # Sort by title, then date, the find the newest date and filter the id.
        output = shellout(""" sort -S30% -t $'\t' -k3,3 -k2,2 < {input} |
                          tac |
                          sort -S30% -t $'\t' -k3,3 -u |
                          cut -f1 | grep -v '^$' > {output} """,
                          input=self.input().path,
                          preserve_whitespace=True)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class DOAJIntermediateSchema(DOAJTask):
    """
    Respect whitelist.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    format = luigi.Parameter(default="doaj-oai",
                             description="kind of source document, doaj-oai (defunkt: doaj, doaj-api)")

    def requires(self):
        return {
            'data': DOAJIntermediateSchemaDirty(date=self.date, format=self.format),
            'whitelist': DOAJWhitelist(date=self.date),
        }

    @timed
    def run(self):
        output = shellout("""unpigz -c {input} | LC_ALL=C grep -Ff {whitelist} | pigz -c > {output}""",
                          whitelist=self.input().get('whitelist').path,
                          input=self.input().get('data').path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'))


class DOAJISSNList(DOAJTask):
    """
    A list of DOAJ ISSNs.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return {
            'input': DOAJIntermediateSchema(date=self.date),
            'jq': Executable(name='jq', message='http://git.io/NYpfTw')
        }

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        shellout("""jq -r '.["rft.issn"][]?' <(unpigz -c {input}) >> {output} """,
                 input=self.input().get('input').path,
                 output=stopover)
        shellout("""jq -r '.["rft.eissn"][]?' <(unpigz -c {input}) >> {output} """,
                 input=self.input().get('input').path,
                 output=stopover)
        output = shellout("""sort -u {input} > {output} """, input=stopover)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class DOAJDOIList(DOAJTask):
    """
    An best-effort list of DOAJ DOIs.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return {
            'input': DOAJIntermediateSchema(date=self.date),
            'jq': Executable(name='jq', message='http://git.io/NYpfTw'),
        }

    @timed
    def run(self):
        output = shellout(
            """jq -r '.doi' <(unpigz -c {input}) | grep -v "null" | grep -o "10.*" 2> /dev/null | sort -u > {output} """,
            input=self.input().get('input').path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)


class DOAJDownloadDump(DOAJTask):
    """
    Download complete set.

    > You can access full data-dumps of the entire DOAJ public metadata below.
    Data dumps are generated weekly. The following files provide you complete
    sets of articles and journals as JSON, in the form that they would also be
    retrieved via the API.

    Data comes in batches of 100k records.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    kind = luigi.Parameter(default='article', description='article or journal')

    def run(self):
        dumps = {
            'article': 'https://doaj.org/public-data-dump/article',
            'journal': 'https://doaj.org/public-data-dump/journal',
        }
        if self.kind not in dumps:
            raise ValueError('kind must be: article or journal')

        original = shellout(""" wget -O {output} {link} """, link=dumps[self.kind])

        # The data comes in batches, each batch is a list of dicts, [{}, {},
        # ...], so we want to flatten all batches into a single json lines file first.
        output = shellout(""" tar -xOzf {input} | jq -rc '.[]' | pigz -c > {output} """, input=original)
        luigi.LocalTarget(output).move(self.output().path)
        os.remove(original)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='gz'), format=Gzip)
